library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.all;
use WORK.cpu_defs.all;

entity ALU is
	port(clk, rst : std_logic;
		  ACC_bus, ACC_load, ALU_ACC, ALU_add,
		  ALU_sub : in std_logic;
		  sysbus : inout std_logic_vector(word_w-1 downto 0);
		  z_flag : out std_logic);
end ALU;

architecture rtl of ALU is
	signal acc : unsigned(word_w-1 downto 0);
	constant zero : unsigned(word_w-1 downto 0) := (others => '0');
begin

sysbus <= std_logic_vector(acc) when ACC_bus = '1' else (others => 'Z');
z_flag <= '1' when acc = zero else '0';

process (clk, rst) is
begin
	if rst = '0' then
		acc <= (others => '0');
	elsif rising_edge(clk) then
		if ACC_load = '1' then
			if ALU_ACC = '1' then
				if ALU_add = '1' then
					acc <= acc + unsigned(sysbus);
				elsif ALU_sub = '1' then
					acc <= acc - unsigned(sysbus);
				end if;
			else
				acc <= unsigned(sysbus);
			end if;
		end if;
	end if;
end process;

end rtl;

