library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.all;
use WORK.cpu_defs.all;

entity RAM is
	port(clk, rst : in std_logic;
		  MDR_bus, MDR_load, MAR_load, CS, R_NW : in std_logic;
		  sysbus : inout std_logic_vector(word_w-1 downto 0));
end RAM;

architecture rtl of RAM is
	signal mdr : std_logic_vector(word_w-1 downto 0);
	signal mar : unsigned(word_w-op_w-1 downto 0);
begin
	sysbus <= mdr when MDR_bus = '1' else (others => 'Z');
	
	process (clk, rst) is 
		type mem_array is array(0 to 2**(word_w-op_w)-1) of
								std_logic_vector(word_w-1 downto 0);
		variable mem : mem_array;
		constant prog : mem_array := (
				0 => op2slv(load) & std_logic_vector(to_unsigned(5, word_w-op_w)),
				1 => op2slv(add) & std_logic_vector(to_unsigned(6, word_w-op_w)),
				2 => op2slv(store) & std_logic_vector(to_unsigned(5, word_w-op_w)),
				3 => op2slv(output) & std_logic_vector(to_unsigned(0, word_w-op_w)),
				4 => op2slv(bne) & std_logic_vector(to_unsigned(7, word_w-op_w)),
				5 => std_logic_vector(to_unsigned(0, word_w)),
				6 => std_logic_vector(to_unsigned(1, word_w)),
				others => (others => '0'));				
	begin
		if rst = '0' then
			mdr <= (others => '0');
			mar <= (others => '0');
			mem := prog;
		elsif rising_edge(clk) then
			if MAR_load = '1' then
				mar <= unsigned(sysbus(word_w-op_w-1 downto 0));
			elsif MDR_load = '1' then
				mdr <= sysbus;
			elsif CS = '1' then
				if R_NW = '1' then
					mdr <= mem(to_integer(mar));
				else
					mem(to_integer(mar)) := mdr;
				end if;
			end if;
		end if;
	end process;
end rtl;

