library IEEE;
use IEEE.STD_LOGIC_1164.all;

package cpu_defs is
	type opcode is (load, store, add, sub, bne, output, jmp);
	constant word_w : natural := 8;	-- szerokosc magistrali
	constant op_w : natural := 3; -- liczba bitow dla kodu operacji
	constant rfill : std_logic_vector(op_w-1 downto 0) := (others => '0');
	
	function slv2op(slv : in std_logic_vector(op_w-1 downto 0)) return opcode;
	function op2slv(op : in opcode) return std_logic_vector;
end cpu_defs;

package body cpu_defs is
	type optable is array(opcode) of std_logic_vector(op_w-1 downto 0);
	constant trans_table : optable := ("000", "001", "010", "011", "100", "101", "110" );
	
	function op2slv(op : in opcode) return std_logic_vector is
	begin
		return trans_table(op);
	end function;
	
	function slv2op(slv : in std_logic_vector(op_w-1 downto 0)) return opcode is
		variable transop : opcode;
	begin		
		case slv is 
			when "000" => transop := load;
			when "001" => transop := store;
			when "010" => transop := add;
			when "011" => transop := sub;
			when "100" => transop := bne;
			when "101" => transop := output;
			when "110" => transop := jmp;
			when others => null;
		end case;
		return transop;
	end function;
end cpu_defs;
