library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use WORK.cpu_defs.all;

entity sequencer is
	port (clk, rst : in std_logic;
			op : in opcode;
			z_flag : in std_logic;
			ACC_bus, ACC_load, PC_bus, PC_load,
			IR_load, MAR_load, MDR_bus, MDR_load, OUTPUT_load,
			ALU_ACC, ALU_add, ALU_sub, PC_inc,
			Addr_bus, CS, R_NW : out std_logic);
end entity;

architecture rtl of sequencer is
	type state is(s0, s1, s2, s3, s4, s4a, s5, s6, s7, s8, s9, s10);
	signal present_state, next_state : state;
begin
seq: process(clk, rst) is
	begin
		if rst = '0' then
			present_state <= s0;
		elsif rising_edge(clk) then
			present_state <= next_state;
		end if;
	end process;

com: process (present_state, op, z_flag) is
	begin
		ACC_bus <= '0';
		ACC_load <= '0';
		PC_bus <= '0';
		PC_load <= '0';
		IR_load <= '0';
		MAR_load <= '0';
		MDR_bus <= '0';
		MDR_load <= '0';
		OUTPUT_load <= '0';
		ALU_ACC <= '0';
		ALU_add <= '0';
		ALU_sub <= '0';
		PC_inc <= '0';
		Addr_bus <= '0';
		CS <= '0';
		R_NW <= '0';
		case present_state is
			when s0 =>
				PC_bus <= '1';
				MAR_load <= '1';
				PC_inc <= '1';
				PC_load <= '1';
				next_state <= s1;
			when s1 =>
				CS <= '1';
				R_NW <= '1';
				next_state <= s2;
			when s2 =>
				MDR_bus <= '1';
				IR_load <= '1';
				next_state <= s3;
			when s3 =>
				Addr_bus <= '1';
				MAR_load <= '1';
				if op = store then
					next_state <= s4;
				elsif op = output then
					next_state <= s4a;
				else
					next_state <= s6;
				end if;
			when s4 =>
				ACC_bus <= '1';
				MDR_load <= '1';
				next_state <= s5;
			when s4a =>
				ACC_bus <= '1';
				OUTPUT_load <= '1';
				next_state <= s0;
			when s5 =>
				CS <= '1';
				next_state <= s0;
			when s6 =>
				CS <= '1';
				R_NW <= '1';
				if op = load then
					next_state <= s7;
				elsif op = bne then
					if z_flag = '0' then
						next_state <= s9;
					else 
						next_state <= s10;
					end if;
				elsif op = jmp then
					next_state <= s10;
				else
					next_state <= s8;
				end if;
			when s7 =>
				MDR_bus <= '1';
				ACC_load <= '1';
				next_state <= s0;
			when s8 =>
				MDR_bus <= '1';
				ALU_ACC <= '1';
				ACC_load <= '1';
				if op = add then
					ALU_add <= '1';
				elsif op = sub then
					ALU_sub <= '1';
				end if;
				next_state <= s0;
			when s9 =>
				MDR_bus <= '1';
				PC_load <= '1';
				next_state <= s0;
			when s10 =>
				next_state <= s0;
		end case;
	end process;

end architecture;

