library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.all;
use WORK.cpu_defs.all;

entity simple_cpu02 is port(
	clk, rst : in std_logic;
	led : out std_logic_vector(7 downto 0));
end entity;

architecture top of simple_cpu02 is

constant CLOCK_DIVIDER : natural := 100000;

signal ACC_bus, ACC_load, PC_bus, PC_load,
		 IR_load, MAR_load, MDR_bus, MDR_load, OUTPUT_load,
		 ALU_ACC, ALU_add, ALU_sub, PC_inc,
		 Addr_bus, CS, R_NW : std_logic;
signal z_flag : std_logic;
signal op : opcode;
signal sysbus : std_logic_vector(word_w-1 downto 0);
signal clk_counter : integer range 0 to CLOCK_DIVIDER := 0;
signal slow_clk : std_logic := '0';

begin	
	process (clk, rst) is
	begin
		if rst = '0' then
			clk_counter <= 0;		
			slow_clk <= '0';
		elsif rising_edge(clk) then
			if clk_counter = CLOCK_DIVIDER-1  then
				slow_clk <= not slow_clk;				
				clk_counter <= 0;
			else
				clk_counter <= clk_counter + 1;
			end if;
		end if;
	end process;

	s1: entity WORK.sequencer port map(
		slow_clk, rst, op, z_flag,
		ACC_bus, ACC_load, PC_bus,
		PC_load, IR_load, MAR_load,
		MDR_bus, MDR_load, OUTPUT_load,
		ALU_ACC, ALU_add, ALU_sub, 
		PC_inc, Addr_bus, CS, R_NW);
	i1: entity WORK.IR port map(
		slow_clk, rst, Addr_bus, IR_load, 
		op, sysbus);
	p1: entity WORK.PC port map(
		slow_clk, rst, PC_bus, PC_load,
		PC_inc, sysbus);		
	a1: entity WORK.ALU port map(
		slow_clk, rst, ACC_bus, ACC_load,
		ALU_ACC, ALU_add, ALU_sub,
		sysbus, z_flag);
	r1: entity WORK.RAM port map(
		slow_clk, rst, MDR_bus, MDR_load,
		MAR_load, CS, R_NW, sysbus);
	o1: entity WORK.OUTPUT port map(
		slow_clk, rst, OUTPUT_load, sysbus, led);
end architecture;

